@@ -23,6 +23,7 @@
 #include <QTimer>
 #include <QToolTip>
 #include <QPainter>
+#include <QDir>
 
 #include <cstdlib>
 #include <fcntl.h>
@@ -105,13 +106,22 @@ MInstall::MInstall(QSettings &acfg, const QCommandLineParser &args, const QStrin
     // ensure the help widgets are displayed correctly when started
     // Qt will delete the heap-allocated event object when posted
     qApp->postEvent(this, new QEvent(QEvent::PaletteChange));
-    QTimer::singleShot(0, this, &MInstall::startup);
+    QTimer::singleShot(0, this, [this]() {
+        try {
+            startup();
+        } catch (const char *msg) {
+            proc.log(QStringLiteral("FAILED START - ") + msg, MProcess::Fail);
+            QMessageBox::critical(this, windowTitle(), tr(msg));
+            exit(EXIT_FAILURE);
+        }
+    });
 }
 
 MInstall::~MInstall() {
     if (oobe) delete oobe;
     if (base) delete base;
     if (bootman) delete bootman;
+    if (swapman) delete swapman;
     if (partman) delete partman;
     if (throbber) delete throbber;
 }
@@ -119,7 +129,7 @@ MInstall::~MInstall() {
 // meant to be run after the installer becomes visible
 void MInstall::startup()
 {
-    proc.log(__PRETTY_FUNCTION__);
+    proc.log(__PRETTY_FUNCTION__, MProcess::Section);
 
     if (!modeOOBE) {
         // Check for a bad combination, like 32-bit ISO and 64-bit UEFI.
@@ -134,16 +144,17 @@ void MInstall::startup()
 
         partman = new PartMan(proc, *this, appConf, appArgs);
         base = new Base(proc, *partman, *this, appConf, appArgs);
+        base->scanMedia();
         bootman = new BootMan(proc, *partman, *this, appConf, appArgs);
+        swapman = new SwapMan(proc, *partman, *this);
 
         ROOT_BUFFER = appConf.value("ROOT_BUFFER", 5000).toInt();
         HOME_BUFFER = appConf.value("HOME_BUFFER", 2000).toInt();
         INSTALL_FROM_ROOT_DEVICE = appConf.value("INSTALL_FROM_ROOT_DEVICE").toBool();
 
-        // Unable to get space requirements since there's no medium.
+        // Unable to get space requirements since there's no media.
         if (!pretend && partman->bootSpaceNeeded==0) {
-            QMessageBox::critical(this, windowTitle(), tr("Cannot access installation source."));
-            exit(EXIT_FAILURE);
+            throw QT_TR_NOOP("Cannot access installation media.");
         }
 
         autoMountEnabled = true; // disable auto mount by force
@@ -171,9 +182,12 @@ void MInstall::startup()
 
     setupkeyboardbutton();
 
-    if (proc.detectMac()) checkLocalClock->setChecked(true);
+    // Guess if hardware clock is UTC or local time
+    proc.shell("guess-hwclock", nullptr, true);
+    if (proc.readOut() == "localtime") checkLocalClock->setChecked(true);
+    else if (proc.detectMac()) checkLocalClock->setChecked(true);
 
-    oobe = new Oobe(proc, *this, this, appConf);
+    oobe = new Oobe(proc, *this, this, appConf, oem, modeOOBE);
 
     if (modeOOBE) manageConfig(ConfigLoadB);
     else {
@@ -186,6 +200,7 @@ void MInstall::startup()
                 item->addToCombo(comboDisk);
             }
         }
+        connect(radioEntireDisk, &QCheckBox::toggled, boxAutoPart, &QGroupBox::setEnabled);
         if (comboDisk->count() >= 1) {
             comboDisk->setCurrentIndex(0);
             radioEntireDisk->setChecked(true);
@@ -204,6 +219,9 @@ void MInstall::startup()
         setupPartitionSlider();
         // Override with whatever is in the config.
         manageConfig(ConfigLoadA);
+        // Hibernation check box (regular install).
+        checkHibernationReg->setChecked(checkHibernation->isChecked());
+        connect(checkHibernationReg, &QCheckBox::clicked, checkHibernation, &QCheckBox::setChecked);
     }
     oobe->stashServices(true);
 
@@ -245,7 +263,7 @@ void MInstall::splashThrob()
 // turn auto-mount off and on
 void MInstall::setupAutoMount(bool enabled)
 {
-    proc.log(__PRETTY_FUNCTION__);
+    proc.log(__PRETTY_FUNCTION__, MProcess::Section);
 
     if (autoMountEnabled == enabled) return;
     QFileInfo finfo;
@@ -266,7 +284,7 @@ void MInstall::setupAutoMount(bool enabled)
     QStringList udev_temp_mdadm_rules;
     finfo.setFile("/run/udev");
     if (finfo.isDir()) {
-        proc.shell("egrep -l '^[^#].*mdadm (-I|--incremental)' /lib/udev/rules.d", nullptr, true);
+        proc.shell("grep -El '^[^#].*mdadm (-I|--incremental)' /lib/udev/rules.d/*", nullptr, true);
         udev_temp_mdadm_rules = proc.readOutLines();
         for (QString &rule : udev_temp_mdadm_rules) {
             rule.replace("/lib/udev", "/run/udev");
@@ -279,7 +297,7 @@ void MInstall::setupAutoMount(bool enabled)
         if (have_sysctl) {
             // Use systemctl to prevent automount by masking currently unmasked mount points
             proc.shell("systemctl list-units --full --all -t mount --no-legend 2>/dev/null"
-                " | grep -v masked | cut -f1 -d' ' | egrep -v '^(dev-hugepages|dev-mqueue|proc-sys-fs-binfmt_misc"
+                " | grep -v masked | cut -f1 -d' ' | grep -Ev '^(dev-hugepages|dev-mqueue|proc-sys-fs-binfmt_misc"
                     "|run-user-.*-gvfs|sys-fs-fuse-connections|sys-kernel-config|sys-kernel-debug)'", nullptr, true);
             const QStringList &maskedMounts = proc.readOutLines();
             if (!maskedMounts.isEmpty()) {
@@ -288,8 +306,8 @@ void MInstall::setupAutoMount(bool enabled)
         }
         // create temporary blank overrides for all udev rules which
         // automatically start Linux Software RAID array members
-        proc.mkpath("/run/udev/rules.d");
-        for (const QString &rule : udev_temp_mdadm_rules) {
+        if (!QDir().exists("/run/udev/rules.d")) proc.mkpath("/run/udev/rules.d");
+        for (const QString &rule : qAsConst(udev_temp_mdadm_rules)) {
             proc.exec("touch", {rule});
         }
 
@@ -307,7 +325,7 @@ void MInstall::setupAutoMount(bool enabled)
             proc.sleep(1000);
         }
         // clear the rules that were temporarily overridden
-        for (const QString &rule : udev_temp_mdadm_rules) {
+        for (const QString &rule : qAsConst(udev_temp_mdadm_rules)) {
             proc.shell("rm -f " + rule); // TODO: check if each rule is a single file name.
         }
 
@@ -374,12 +392,8 @@ bool MInstall::processNextPhase()
 
             // the core of the installation
             if (!pretend) {
-                proc.advance(12, partman->countPrepSteps());
-                partman->preparePartitions();
-                partman->formatPartitions();
-                partman->mountPartitions();
-                //run blkid -c /dev/null to freshen UUID cache
-                proc.exec("blkid", {"-c", "/dev/null"});
+                proc.advance(11, partman->countPrepSteps());
+                partman->prepStorage();
                 base->install();
             } else {
                 if (!pretendToInstall(14, 200)) return false;
@@ -404,7 +418,9 @@ bool MInstall::processNextPhase()
                 manageConfig(ConfigSave);
                 config->dumpDebug();
                 proc.exec("/bin/sync"); // the sync(2) system call will block the GUI
+                swapman->install();
                 bootman->install();
+                if (oem) proc.shell("sed -i 's/splash\\b/nosplash/g' /mnt/antiX/boot/grub/grub.cfg");
             } else if (!pretendToInstall(5, 100)) {
                 return false;
             }
@@ -413,6 +429,12 @@ bool MInstall::processNextPhase()
             proc.status(tr("Cleaning up"));
             cleanup();
             proc.status(tr("Finished"));
+            if (!pretend && appArgs.isSet("reboot")) {
+                proc.shell("/usr/local/bin/persist-config --shutdown --command reboot &");
+            }
+            if (!pretend && appArgs.isSet("poweroff")) {
+                proc.shell("/usr/local/bin/persist-config --shutdown --command poweroff &");
+            }
             gotoPage(Step::End);
         }
     } catch (const char *msg) {
@@ -452,12 +474,15 @@ void MInstall::manageConfig(enum ConfigAction mode)
         if (targetDrive || mode!=ConfigSave) {
             config->manageComboBox("Drive", comboDisk, true);
             config->manageGroupCheckBox("DriveEncrypt", boxEncryptAuto);
-            if (mode==ConfigSave) config->setValue("RootPortion", sliderPart->value());
-            else if (config->contains("RootPortion")) {
+            if (mode == ConfigSave) {
+                config->setValue("RootPortion", sliderPart->value());
+            } else if (config->contains("RootPortion")) {
                  const int sliderVal = config->value("RootPortion").toInt();
                  sliderPart->setValue(sliderVal);
                  on_sliderPart_valueChanged(sliderVal);
-                 if (sliderPart->value() != sliderVal) config->markBadWidget(sliderPart);
+                 if (sliderPart->value() != sliderVal) {
+                     config->markBadWidget(sliderPart);
+                 }
             }
         }
         config->endGroup();
@@ -483,10 +508,10 @@ void MInstall::manageConfig(enum ConfigAction mode)
         config->endGroup();
     }
 
+    const bool confAdvanced = (!modeOOBE && radioCustomPart->isChecked());
+    if (confAdvanced) swapman->manageConfig(*config);
     if (mode == ConfigSave || mode == ConfigLoadB) {
-        // GRUB page
-        if (!modeOOBE) bootman->manageConfig(*config);
-        // Manage the rest of the OOBE pages.
+        if (confAdvanced) bootman->manageConfig(*config);
         oobe->manageConfig(*config, mode==ConfigSave);
     }
 
@@ -505,7 +530,7 @@ void MInstall::manageConfig(enum ConfigAction mode)
 
 bool MInstall::saveHomeBasic()
 {
-    proc.log(__PRETTY_FUNCTION__);
+    proc.log(__PRETTY_FUNCTION__, MProcess::Section);
     QString homedir("/");
     QString homedev = partman->getMountDev("/home", true);
     if (homedev.isEmpty() || partman->willFormat("/home")) {
@@ -521,18 +546,23 @@ bool MInstall::saveHomeBasic()
     // Store a listing of /home to compare with the user name given later.
     mkdir("/mnt/antiX", 0755);
     bool ok = proc.exec("/bin/mount", {"-o", "ro", homedev, "/mnt/antiX"});
-    if (ok) ok = proc.exec("/bin/ls", {"-1", "/mnt/antiX" + homedir}, nullptr, true);
-    if (ok) listHomes = proc.readOutLines();
-    proc.exec("/usr/bin/umount", {"-l", "/mnt/antiX"});
+    if (ok) {
+        QDir hd("/mnt/antiX" + homedir);
+        ok = hd.exists() && hd.isReadable();
+        listHomes = hd.entryList(QDir::Dirs);
+        proc.exec("/usr/bin/umount", {"-l", "/mnt/antiX"});
+    }
     return ok;
 }
 
 // logic displaying pages
 int MInstall::showPage(int curr, int next)
 {
-    if (next == Step::Splash) splashSetThrobber(true); // Enter
-    else if (curr == Step::Splash) splashSetThrobber(false); // Leave
-    else if (curr == Step::Disk && next > curr) {
+    if (next == Step::Splash) { // Enter splash screen
+        splashSetThrobber(appConf.value("SPLASH_THROBBER", true).toBool());
+    } else if (curr == Step::Splash) { // Leave splash screen
+        splashSetThrobber(false);
+    } else if (curr == Step::Disk && next > curr) {
         if (radioEntireDisk->isChecked()) {
             if (!automatic) {
                 QString msg = tr("OK to format and use the entire disk (%1) for %2?");
@@ -555,7 +585,11 @@ int MInstall::showPage(int curr, int next)
                 nextFocus = treePartitions;
                 return curr;
             }
-            return Step::Boot;
+            bootman->buildBootLists(); // Load default boot options
+            swapman->setupDefaults();
+            manageConfig(ConfigLoadB);
+            checkHibernation->setChecked(checkHibernationReg->isChecked());
+            return Step::Network;
         }
     } else if (curr == Step::Partitions && next > curr) {
         if (!partman->composeValidate(automatic, PROJECTNAME)) {
@@ -613,6 +647,7 @@ int MInstall::showPage(int curr, int next)
 
 void MInstall::pageDisplayed(int next)
 {
+    bool enableBack = true, enableNext = true;
     if (!modeOOBE) {
         const int ixProgress = widgetStack->indexOf(pageProgress);
         // progress bar shown only for install and configuration pages.
@@ -668,10 +703,9 @@ void MInstall::pageDisplayed(int next)
             proc.unhalt();
             updateCursor();
         }
-        pushBack->setEnabled(true);
-        pushNext->setEnabled(radioCustomPart->isChecked()
-            || !boxEncryptAuto->isChecked() || textCryptoPass->isValid());
-        return; // avoid the end that enables both Back and Next buttons
+        enableNext = radioCustomPart->isChecked()
+            || !boxEncryptAuto->isChecked() || textCryptoPass->isValid();
+        break;
 
     case Step::Partitions:
         textHelp->setText("<p><b>" + tr("Choose Partitions") + "</b><br/>"
@@ -755,23 +789,27 @@ void MInstall::pageDisplayed(int next)
             "<p><b>" + tr("Virtual Devices") + "</b><br/>"
             + tr("If the intaller detects any virtual devices such as opened LUKS partitions, LVM logical volumes or software-based RAID volumes, they may be used for the installation.") + "</p>"
             "<p>" + tr("The use of virtual devices (beyond preserving encrypted file systems) is an advanced feature. You may have to edit some files (eg. initramfs, crypttab, fstab) to ensure the virtual devices used are created upon boot.") + "</p>");
-        pushBack->setEnabled(true);
-        pushNext->setEnabled(!(boxCryptoPass->isEnabledTo(boxCryptoPass->parentWidget())) || textCryptoPassCust->isValid());
-        return; // avoid the end that enables both Back and Next buttons
+        enableNext = !(boxCryptoPass->isEnabledTo(boxCryptoPass->parentWidget())) || textCryptoPassCust->isValid();
+        break;
 
     case Step::Boot: // Start of installation.
-        textHelp->setText(tr("<p><b>Select Boot Method</b><br/> %1 uses the GRUB bootloader to boot %1 and MS-Windows. "
-                             "<p>By default GRUB2 is installed in the Master Boot Record (MBR) or ESP (EFI System Partition for 64-bit UEFI boot systems) of your boot drive and replaces the boot loader you were using before. This is normal.</p>"
-                             "<p>If you choose to install GRUB2 to Partition Boot Record (PBR) instead, then GRUB2 will be installed at the beginning of the specified partition. This option is for experts only.</p>"
-                             "<p>If you uncheck the Install GRUB box, GRUB will not be installed at this time. This option is for experts only.</p>").arg(PROJECTNAME));
+        textHelp->setText("<p><b>" + tr("Install GRUB for Linux and Windows") + "</b><br/>"
+            + tr("%1 uses the GRUB bootloader to boot %1 and Microsoft Windows.").arg(PROJECTNAME) + "</p>"
+            "<p>" + tr("By default GRUB is installed in the Master Boot Record (MBR) or ESP (EFI System Partition for 64-bit UEFI boot systems) of your boot drive and replaces the boot loader you were using before. This is normal.") + "</p>"
+            "<p>" + tr("If you choose to install GRUB to Partition Boot Record (PBR) instead, then GRUB will be installed at the beginning of the specified partition. This option is for experts only.") + "</p>"
+            "<p>" + tr("If you uncheck the Install GRUB box, GRUB will not be installed at this time. This option is for experts only.") + "</p>"
+            "<p><b>" + tr("Create a swap file") + "</b><br/>"
+            + tr("A swap file is more flexible than a swap partition; it is considerably easier to resize a swap file to adapt to changes in system usage.") + "</p>"
+            "<p>" + tr("By default, this is checked if no swap partitions have been set, and unchecked if swap partitions are set. This option should be left untouched, and is for experts only.") + "<br/>"
+            + tr("Setting the size to 0 has the same effect as unchecking this option.") + "</p>");
 
-        pushBack->setEnabled(false);
-        pushNext->setEnabled(true);
+        enableBack = false;
         if (phase <= 0) {
             bootman->buildBootLists();
+            swapman->setupDefaults();
             manageConfig(ConfigLoadB);
         }
-        return; // avoid the end that enables both Back and Next buttons
+        break;
 
     case Step::Services:
         textHelp->setText(tr("<p><b>Common Services to Enable</b><br/>Select any of these common services that you might need with your system configuration and the services will be started automatically when you start %1.</p>").arg(PROJECTNAME));
@@ -783,11 +821,8 @@ void MInstall::pageDisplayed(int next)
                              "<p>The computer and domain names can contain only alphanumeric characters, dots, hyphens. They cannot contain blank spaces, start or end with hyphens</p>"
                              "<p>The SaMBa Server needs to be activated if you want to use it to share some of your directories or printer "
                              "with a local computer that is running MS-Windows or Mac OSX.</p>"));
-        if (modeOOBE) {
-            pushBack->setEnabled(false);
-            pushNext->setEnabled(true);
-            return; // avoid the end that enables both Back and Next buttons
-        }
+        if (modeOOBE) enableBack = false;
+        else enableBack = radioCustomPart->isChecked();
         break;
 
     case Step::Localization:
@@ -819,9 +854,8 @@ void MInstall::pageDisplayed(int next)
         + tr("Obviously, this should only be done in situations where the user account"
             " does not need to be secure, such as a public terminal.") + "</p>");
         if (!nextFocus) nextFocus = textUserName;
-        pushBack->setEnabled(true);
         oobe->userPassValidationChanged();
-        return; // avoid the end that enables both Back and Next buttons
+        break;
 
     case Step::OldHome:
         textHelp->setText("<p><b>" + tr("Old Home Directory") + "</b><br/>"
@@ -842,9 +876,9 @@ void MInstall::pageDisplayed(int next)
         // disable the Next button if none of the old home options are selected
         oobe->oldHomeToggled();
         // if the Next button is disabled, avoid enabling both Back and Next at the end
-        if (pushNext->isEnabled() == false) {
-            pushBack->setEnabled(true);
-            return;
+        if (!pushNext->isEnabled()) {
+            enableBack = true;
+            enableNext = false;
         }
         break;
 
@@ -863,9 +897,8 @@ void MInstall::pageDisplayed(int next)
             + "</p><p>"
             + tr("Complete these steps at your own pace. The installer will wait for your input if necessary.")
             + "</p>");
-        pushBack->setEnabled(true);
-        pushNext->setEnabled(false);
-        return; // avoid enabling both Back and Next buttons at the end
+        enableNext = false;
+        break;
 
     case Step::End:
         pushClose->setEnabled(false);
@@ -878,17 +911,17 @@ void MInstall::pageDisplayed(int next)
         break;
 
     default: // other
-        textHelp->setText("<p><b>" + tr("Enjoy using %1</b></p>").arg(PROJECTNAME) + "\n\n "
+        textHelp->setText("<p><b>" + tr("Enjoy using %1").arg(PROJECTNAME) + "</b></p>"
         + tr("<p><b>Support %1</b><br/>"
             "%1 is supported by people like you. Some help others at the "
             "support forum - %2 - or translate help files into different "
-            "languages, or make suggestions, write documentation, or help test new software.</p>").arg(PROJECTNAME).arg(PROJECTFORUM));
+            "languages, or make suggestions, write documentation, or help test new software.</p>").arg(PROJECTNAME, PROJECTFORUM));
         pushNext->setDefault(true);
         break;
     }
 
-    pushBack->setEnabled(true);
-    pushNext->setEnabled(true);
+    pushBack->setEnabled(enableBack);
+    pushNext->setEnabled(enableNext);
 }
 
 void MInstall::gotoPage(int next)
@@ -940,13 +973,16 @@ void MInstall::gotoPage(int next)
 
     // automatic installation
     if (automatic) {
-        if (!MSettings::isBadWidget(widgetStack->currentWidget())
-            && next > curr) pushNext->click();
-        else if (curr!=0) automatic = false; // failed validation
+        if (!MSettings::isBadWidget(widgetStack->currentWidget()) && next > curr) {
+            pushNext->click();
+        } else if (curr!=0) { // failed validation
+            automatic = false;
+        }
     }
 
     // process next installation phase
-    if (next == widgetStack->indexOf(pageBoot) || next == widgetStack->indexOf(pageProgress)) {
+    if (next == Step::Boot || next == Step::Progress
+        || (radioEntireDisk->isChecked() && next == Step::Network)) {
         if (modeOOBE) {
             updateCursor(Qt::BusyCursor);
             labelSplash->setText(tr("Configuring sytem. Please wait."));
@@ -1034,9 +1070,7 @@ bool MInstall::eventFilter(QObject *watched, QEvent *event)
 void MInstall::changeEvent(QEvent *event)
 {
     const QEvent::Type etype = event->type();
-    if (etype == QEvent::ApplicationPaletteChange
-        || etype == QEvent::PaletteChange || etype == QEvent::StyleChange)
-    {
+    if (etype == QEvent::ApplicationPaletteChange || etype == QEvent::PaletteChange || etype == QEvent::StyleChange) {
         QPalette pal = qApp->palette(textHelp);
         QColor col = pal.color(QPalette::Base);
         col.setAlpha(200);
@@ -1051,6 +1085,7 @@ void MInstall::closeEvent(QCloseEvent *event)
     if (abortUI()) {
         event->accept();
         phase = -2;
+        if (base) base->haltCheck(true);
         if (!modeOOBE) cleanup();
         else if (!pretend) {
             proc.unhalt();
@@ -1068,9 +1103,10 @@ void MInstall::closeEvent(QCloseEvent *event)
     }
 }
 
+// Override QDialog::reject() so Escape won't close the window.
 void MInstall::reject()
 {
-    // dummy (overrides QDialog::reject() so Escape won't close the window)
+    if (base) base->haltCheck(false);
 }
 
 /////////////////////////////////////////////////////////////////////////
@@ -1110,13 +1146,12 @@ void MInstall::on_pushServices_clicked()
 
 bool MInstall::abortUI()
 {
-    proc.log(__PRETTY_FUNCTION__);
+    proc.log(__PRETTY_FUNCTION__, MProcess::Section);
     boxMain->setEnabled(false);
     // ask for confirmation when installing (except for some steps that don't need confirmation)
     if (phase > 0 && phase < 4) {
-        const QMessageBox::StandardButton rc = QMessageBox::warning(this,
-            tr("Confirmation"), tr("The installation and configuration"
-                " is incomplete.\nDo you really want to stop now?"),
+        const QMessageBox::StandardButton rc = QMessageBox::warning(this, QString(),
+            tr("The installation and configuration is incomplete.\nDo you really want to stop now?"),
             QMessageBox::Yes | QMessageBox::No, QMessageBox::No);
         if (rc == QMessageBox::No) {
             boxMain->setEnabled(true);
@@ -1132,7 +1167,7 @@ bool MInstall::abortUI()
 // run before closing the app, do some cleanup
 void MInstall::cleanup(bool endclean)
 {
-    proc.log(__PRETTY_FUNCTION__);
+    proc.log(__PRETTY_FUNCTION__, MProcess::Section);
     if (pretend) return;
 
     if (endclean) {
@@ -1141,25 +1176,27 @@ void MInstall::cleanup(bool endclean)
         proc.exec("/usr/bin/cp", {"/var/log/minstall.log", "/mnt/antiX/var/log"});
         proc.exec("/usr/bin/rm", {"-rf", "/mnt/antiX/mnt/antiX"});
     }
-    proc.exec("/usr/bin/umount", {"-lq", "/mnt/antiX/boot/efi"});
-    proc.exec("/usr/bin/umount", {"-lq", "/mnt/antiX/proc"});
-    proc.exec("/usr/bin/umount", {"-lq", "/mnt/antiX/sys"});
-    proc.exec("/usr/bin/umount", {"-lq", "/mnt/antiX/dev/shm"});
-    proc.exec("/usr/bin/umount", {"-lq", "/mnt/antiX/dev"});
+    if (proc.exec("/usr/bin/mountpoint", {"-q", "/mnt/antiX/boot/efi"})) proc.exec("/usr/bin/umount", {"-lq", "/mnt/antiX/boot/efi"});
+    if (proc.exec("/usr/bin/mountpoint", {"-q", "/mnt/antiX/proc"}))     proc.exec("/usr/bin/umount", {"-lq", "/mnt/antiX/proc"});
+    if (proc.exec("/usr/bin/mountpoint", {"-q", "/mnt/antiX/sys"}))      proc.exec("/usr/bin/umount", {"-lq", "/mnt/antiX/sys"});
+    if (proc.exec("/usr/bin/mountpoint", {"-q", "/mnt/antiX/dev/shm"}))  proc.exec("/usr/bin/umount", {"-lq", "/mnt/antiX/dev/shm"});
+    if (proc.exec("/usr/bin/mountpoint", {"-q", "/mnt/antiX/dev"}))      proc.exec("/usr/bin/umount", {"-lq", "/mnt/antiX/dev"});
     if (endclean && !mountkeep) partman->unmount();
 }
 
 void MInstall::on_progInstall_valueChanged(int value)
 {
-    if (ixTipStart < 0 || widgetStack->currentWidget() != pageProgress)
+    if (ixTipStart < 0 || widgetStack->currentWidget() != pageProgress) {
         return; // no point displaying a new hint if it will be invisible
+    }
 
     const int tipcount = 6;
     ixTip = tipcount;
     if (ixTipStart < tipcount) {
         int imax = (progInstall->maximum() - iLastProgress) / (tipcount - ixTipStart);
-        if (imax != 0)
+        if (imax != 0) {
             ixTip = ixTipStart + (value - iLastProgress) / imax;
+        }
     }
 
     switch(ixTip)
@@ -1169,7 +1206,7 @@ void MInstall::on_progInstall_valueChanged(int value)
                              "Basic information about %1 is at %2.</p><p>"
                              "There are volunteers to help you at the %3 forum, %4</p>"
                              "<p>If you ask for help, please remember to describe your problem and your computer "
-                             "in some detail. Usually statements like 'it didn't work' are not helpful.</p>").arg(PROJECTNAME).arg(PROJECTURL).arg(PROJECTSHORTNAME).arg(PROJECTFORUM));
+                             "in some detail. Usually statements like 'it didn't work' are not helpful.</p>").arg(PROJECTNAME, PROJECTURL, PROJECTSHORTNAME, PROJECTFORUM));
         break;
 
     case 1:
@@ -1182,7 +1219,7 @@ void MInstall::on_progInstall_valueChanged(int value)
         textTips->setText(tr("<p><b>Support %1</b><br/>"
                              "%1 is supported by people like you. Some help others at the "
                              "support forum - %2 - or translate help files into different "
-                             "languages, or make suggestions, write documentation, or help test new software.</p>").arg(PROJECTNAME).arg(PROJECTFORUM));
+                             "languages, or make suggestions, write documentation, or help test new software.</p>").arg(PROJECTNAME, PROJECTFORUM));
 
         break;
 
@@ -1196,13 +1233,13 @@ void MInstall::on_progInstall_valueChanged(int value)
 
     case 4:
         textTips->setText(tr("<p><b>Keep Your Copy of %1 up-to-date</b><br/>"
-                             "For more information and updates please visit</p><p> %2</p>").arg(PROJECTNAME).arg(PROJECTFORUM));
+                             "For more information and updates please visit</p><p> %2</p>").arg(PROJECTNAME, PROJECTFORUM));
         break;
 
     default:
         textTips->setText(tr("<p><b>Special Thanks</b><br/>Thanks to everyone who has chosen to support %1 with their time, money, suggestions, work, praise, ideas, promotion, and/or encouragement.</p>"
                              "<p>Without you there would be no %1.</p>"
-                             "<p>%2 Dev Team</p>").arg(PROJECTNAME).arg(PROJECTSHORTNAME));
+                             "<p>%2 Dev Team</p>").arg(PROJECTNAME, PROJECTSHORTNAME));
         break;
     }
 }
@@ -1215,22 +1252,21 @@ void MInstall::on_pushClose_clicked()
 void MInstall::setupkeyboardbutton()
 {
     QFile file("/etc/default/keyboard");
-    if (file.open(QFile::ReadOnly | QFile::Text)) {
-        while (!file.atEnd()) {
-            QString line(file.readLine().trimmed());
-            QLabel *plabel = nullptr;
-            if (line.startsWith("XKBMODEL")) plabel = labelKeyboardModel;
-            else if (line.startsWith("XKBLAYOUT")) plabel = labelKeyboardLayout;
-            else if (line.startsWith("XKBVARIANT")) plabel = labelKeyboardVariant;
-            if (plabel != nullptr) {
-                line = line.section('=', 1);
-                line.replace(",", " ");
-                line.remove(QChar('"'));
-                plabel->setText(line);
-            }
+    if (!file.open(QFile::ReadOnly | QFile::Text)) return;
+    while (!file.atEnd()) {
+        QString line(file.readLine().trimmed());
+        QLabel *plabel = nullptr;
+        if (line.startsWith("XKBMODEL")) plabel = labelKeyboardModel;
+        else if (line.startsWith("XKBLAYOUT")) plabel = labelKeyboardLayout;
+        else if (line.startsWith("XKBVARIANT")) plabel = labelKeyboardVariant;
+        if (plabel != nullptr) {
+            line = line.section('=', 1);
+            line.replace(",", " ");
+            line.remove(QChar('"'));
+            plabel->setText(line);
         }
-        file.close();
     }
+    file.close();
 }
 
 void MInstall::on_pushSetKeyboard_clicked()
@@ -1258,7 +1294,9 @@ void MInstall::on_sliderPart_sliderPressed()
     else if (val<1) tipText = tipText.arg(">0", "<100");
     else tipText = tipText.arg(val).arg(100-val);
     sliderPart->setToolTip(tipText);
-    if (sliderPart->isSliderDown()) QToolTip::showText(QCursor::pos(), tipText, sliderPart);
+    if (sliderPart->isSliderDown()) {
+        QToolTip::showText(QCursor::pos(), tipText, sliderPart);
+    }
 }
 void MInstall::on_sliderPart_valueChanged(int value)
 {
