@@ -20,6 +20,7 @@
  *
  * This file is part of the gazelle-installer.
  ***************************************************************************/
+#include <sys/stat.h>
 #include <QDebug>
 #include <QMessageBox>
 #include <QFileInfo>
@@ -27,8 +28,8 @@
 #include <QTimeZone>
 #include "oobe.h"
 
-Oobe::Oobe(MProcess &mproc, Ui::MeInstall &ui, QWidget *parent, const QSettings &appConf)
-    : QObject(parent), proc(mproc), gui(ui), master(parent)
+Oobe::Oobe(MProcess &mproc, Ui::MeInstall &ui, QWidget *parent, const QSettings &appConf, bool oem, bool modeOOBE)
+    : QObject(parent), proc(mproc), gui(ui), master(parent), oem(oem), online(modeOOBE)
 {
     gui.textComputerName->setText(appConf.value("DEFAULT_HOSTNAME").toString());
     enableServices = appConf.value("ENABLE_SERVICES").toStringList();
@@ -51,7 +52,7 @@ Oobe::Oobe(MProcess &mproc, Ui::MeInstall &ui, QWidget *parent, const QSettings
             "-mindepth", "2", "-type", "f", "-printf", "%P\\n"}, nullptr, true);
     timeZones = proc.readOutLines();
     gui.comboTimeArea->clear();
-    for (const QString &zone : timeZones) {
+    for (const QString &zone : qAsConst(timeZones)) {
         const QString &area = zone.section('/', 0, 0);
         if (gui.comboTimeArea->findData(QVariant(area)) < 0) {
             QString text(area);
@@ -194,32 +195,36 @@ void Oobe::enable()
     proc.setChRoot();
 }
 
-bool Oobe::process()
+void Oobe::process()
 {
-    if (!online) proc.setChRoot("/mnt/antiX");
-    QTreeWidgetItemIterator it(gui.treeServices);
-    for (; *it; ++it) {
-        if ((*it)->parent()) setService((*it)->text(0), (*it)->checkState(0) == Qt::Checked);
-    }
-    if (haveSamba) {
-        const bool enable = gui.checkSamba->isChecked();
-        setService("smbd", enable);
-        setService("nmbd", enable);
-        setService("samba-ad-dc", enable);
-    }
-    proc.setChRoot();
+    if (!oem) {
+        if (!online) proc.setChRoot("/mnt/antiX");
 
-    if (!setComputerName()) return false;
-    setLocale();
-    if (haveSnapshotUserAccounts) { // skip user account creation
+        QTreeWidgetItemIterator it(gui.treeServices);
+        for (; *it; ++it) {
+            if ((*it)->parent()) setService((*it)->text(0), (*it)->checkState(0) == Qt::Checked);
+        }
+        if (haveSamba) {
+            const bool enable = gui.checkSamba->isChecked();
+            setService("smbd", enable);
+            setService("nmbd", enable);
+            setService("samba-ad-dc", enable);
+        }
+        proc.setChRoot();
+        setComputerName();
+        setLocale();
+    }
+    if (haveSnapshotUserAccounts || oem) { // skip user account creation
         proc.exec("rsync", {"-a", "/home/", "/mnt/antiX/home/",
             "--exclude", ".cache", "--exclude", ".gvfs", "--exclude", ".dbus", "--exclude", ".Xauthority",
             "--exclude", ".ICEauthority", "--exclude", ".config/session"});
     } else {
-        if (!setUserInfo()) return false;
+        setUserInfo();
+    }
+    if (online) {
+        proc.shell("sed -i 's/nosplash\\b/splash/g' /boot/grub/grub.cfg");
+        proc.exec("update-rc.d", {"oobe", "disable"});
     }
-    if (online) proc.exec("update-rc.d", {"oobe", "disable"});
-    return true;
 }
 
 /* Services */
@@ -233,7 +238,7 @@ void Oobe::buildServiceList()
     QSettings services_desc("/usr/share/gazelle-installer-data/services.list", QSettings::NativeFormat);
     services_desc.setIniCodec("UTF-8");
 
-    for (const QString &service : enableServices) {
+    for (const QString &service : qAsConst(enableServices)) {
         const QString &lang = QLocale::system().bcp47Name().toLower();
         QString lang_str = (lang == "en")? "" : "_" + lang;
         QStringList list = services_desc.value(service + lang_str).toStringList();
@@ -284,7 +289,7 @@ void Oobe::setService(const QString &service, bool enabled)
 {
     qDebug() << "Set service:" << service << enabled;
     if (enabled) {
-        proc.exec("update-rc.d", {service, " defaults"});
+        proc.exec("update-rc.d", {service, "defaults"});
         if (containsSystemD) proc.exec("systemctl", {"enable", service});
         if (containsRunit) {
             QFile::remove(proc.chRoot + "/etc/sv/" + service + "/down");
@@ -344,7 +349,7 @@ QWidget *Oobe::validateComputerName()
 }
 
 // set the computer name, can not be rerun
-bool Oobe::setComputerName()
+void Oobe::setComputerName()
 {
     QString etcpath = online ? "/etc" : "/mnt/antiX/etc";
     if (haveSamba) {
@@ -365,7 +370,6 @@ bool Oobe::setComputerName()
     proc.shell("sed -i 's/.*send host-name.*/send host-name \""
         + compname + "\";/g' " + etcpath + "/dhcp/dhclient.conf");
     proc.shell("echo \"" + compname + "\" | cat > " + etcpath + "/defaultdomain");
-    return true;
 }
 
 // return 0 = success, 1 = bad area, 2 = bad zone
@@ -383,16 +387,14 @@ int Oobe::selectTimeZone(const QString &zone)
 
 void Oobe::setLocale()
 {
-    proc.log(__PRETTY_FUNCTION__);
-    QString cmd2;
-    QString cmd;
+    proc.log(__PRETTY_FUNCTION__, MProcess::Section);
 
     //locale
+    qDebug() << "Update locale";
+    QString cmd;
     if (!online) cmd = "chroot /mnt/antiX ";
     cmd += QString("/usr/sbin/update-locale \"LANG=%1\"").arg(gui.comboLocale->currentData().toString());
-    qDebug() << "Update locale";
     proc.shell(cmd);
-    cmd = QString("Language=%1").arg(gui.comboLocale->currentData().toString());
     const QString &selTimeZone = gui.comboTimeZone->currentData().toString();
 
     // /etc/localtime is either a file or a symlink to a file in /usr/share/zoneinfo. Use the one selected by the user.
@@ -413,8 +415,8 @@ void Oobe::setLocale()
     }
     proc.exec("hwclock", {"--hctosys"});
     if (!online) {
-        proc.exec("/bin/cp", {"-f /etc/adjtime", "/mnt/antiX/etc/"});
-        proc.exec("/bin/cp", {"-f /etc/default/rcS", "/mnt/antiX/etc/default"});
+        proc.exec("/bin/cp", {"-f", "/etc/adjtime", "/mnt/antiX/etc/"});
+        proc.exec("/bin/cp", {"-f", "/etc/default/rcS", "/mnt/antiX/etc/default"});
     }
 
     // Set clock format
@@ -425,11 +427,17 @@ void Oobe::setLocale()
         proc.shell("sed -i '/data0=/c\\data0=%l:%M' " + skelpath + "/.config/xfce4/panel/xfce4-orageclock-plugin-1.rc");
         proc.shell("sed -i '/time_format=/c\\time_format=%l:%M' /home/demo/.config/xfce4/panel/datetime-1.rc");
         proc.shell("sed -i '/time_format=/c\\time_format=%l:%M' " + skelpath + "/.config/xfce4/panel/datetime-1.rc");
+        proc.shell("sed -i 's/%H/%l/' /home/demo/.config/xfce4/xfconf/xfce-perchannel-xml/xfce4-panel.xml");
+        proc.shell("sed -i 's/%H/%l/' " + skelpath + "/.config/xfce4/xfconf/xfce-perchannel-xml/xfce4-panel.xml");
 
         //mx kde
         proc.shell("sed -i '/use24hFormat=/c\\use24hFormat=0' /home/demo/.config/plasma-org.kde.plasma.desktop-appletsrc");
         proc.shell("sed -i '/use24hFormat=/c\\use24hFormat=0' " + skelpath + "/.config/plasma-org.kde.plasma.desktop-appletsrc");
 
+        //mx fluxbox
+        proc.shell("sed -i '/time1_format/c\\time1_format=%l:%M' " + skelpath + "/.config/tint2/tint2rc");
+        proc.shell("sed -i '/time1_format/c\\time1_format=%l:%M' /home/demo/.config/tint2/tint2rc");
+
         //antix systems
         proc.shell("sed -i 's/%H:%M/%l:%M/g' " + skelpath + "/.icewm/preferences");
         proc.shell("sed -i 's/%k:%M/%l:%M/g' " + skelpath + "/.fluxbox/init");
@@ -440,11 +448,18 @@ void Oobe::setLocale()
         proc.shell("sed -i '/data0=/c\\data0=%H:%M' " + skelpath + "/.config/xfce4/panel/xfce4-orageclock-plugin-1.rc");
         proc.shell("sed -i '/time_format=/c\\time_format=%H:%M' /home/demo/.config/xfce4/panel/datetime-1.rc");
         proc.shell("sed -i '/time_format=/c\\time_format=%H:%M' " + skelpath + "/.config/xfce4/panel/datetime-1.rc");
+        proc.shell("sed -i 's/%l/%H/' /home/demo/.config/xfce4/xfconf/xfce-perchannel-xml/xfce4-panel.xml");
+        proc.shell("sed -i 's/%l/%H/' " + skelpath + "/.config/xfce4/xfconf/xfce-perchannel-xml/xfce4-panel.xml");
+
 
         //mx kde
         proc.shell("sed -i '/use24hFormat=/c\\use24hFormat=2' /home/demo/.config/plasma-org.kde.plasma.desktop-appletsrc");
         proc.shell("sed -i '/use24hFormat=/c\\use24hFormat=2' " + skelpath + "/.config/plasma-org.kde.plasma.desktop-appletsrc");
 
+        //mx fluxbox
+        proc.shell("sed -i '/time1_format/c\\time1_format=%H:%M' " + skelpath + "/.config/tint2/tint2rc");
+        proc.shell("sed -i '/time1_format/c\\time1_format=%H:%M' /home/demo/.config/tint2/tint2rc");
+
         //antix systems
         proc.shell("sed -i 's/%H:%M/%H:%M/g' " + skelpath + "/.icewm/preferences");
         proc.shell("sed -i 's/%k:%M/%k:%M/g' " + skelpath + "/.fluxbox/init");
@@ -492,7 +507,7 @@ QWidget *Oobe::validateUserInfo(bool automatic)
 }
 
 // setup the user, cannot be rerun
-bool Oobe::setUserInfo()
+void Oobe::setUserInfo()
 {
     // set the user passwords first
     bool ok = true;
@@ -512,8 +527,7 @@ bool Oobe::setUserInfo()
     }
     if (ok && !userinfo.isEmpty()) ok = proc.exec("chpasswd", {}, &userinfo);
     if (!ok) {
-        failure = tr("Failed to set user account passwords.");
-        return false;
+        throw QT_TR_NOOP("Failed to set user account passwords.");
     }
     proc.setChRoot();
 
@@ -525,20 +539,18 @@ bool Oobe::setUserInfo()
 
     if (QFileInfo::exists(dpath)) {
         if (gui.radioOldHomeSave->isChecked()) {
-            bool ok = false;
+            ok = false;
             QStringList cargs({"-f", dpath, dpath});
             for (int ixi = 1; ixi < 10 && !ok; ++ixi) {
                 cargs.last() = dpath + ".00" + QString::number(ixi);
                 ok = proc.exec("/bin/mv", cargs);
             }
             if (!ok) {
-                failure = tr("Failed to save old home directory.");
-                return false;
+                throw QT_TR_NOOP("Failed to save old home directory.");
             }
         } else if (gui.radioOldHomeDelete->isChecked()) {
             if (!proc.exec("/bin/rm", {"-rf", dpath})) {
-                failure = tr("Failed to delete old home directory.");
-                return false;
+                throw QT_TR_NOOP("Failed to delete old home directory.");
             }
         }
         proc.exec("/bin/sync"); // The sync(2) system call will block the GUI.
@@ -559,13 +571,11 @@ bool Oobe::setUserInfo()
         // Copy skel to demo, unless demo folder exists in remastered linuxfs.
         if (!remasteredDemo) {
             if (!proc.exec("/bin/cp", {"-a", skelpath, dpath})) {
-                failure = tr("Sorry, failed to create user directory.");
-                return false;
+                throw QT_TR_NOOP("Sorry, failed to create user directory.");
             }
         } else { // still rename the demo directory even if remastered demo home folder is detected
             if (!proc.exec("/bin/mv", {"-f", rootpath + "/home/demo", dpath})) {
-                failure = tr("Sorry, failed to name user directory.");
-                return false;
+                throw QT_TR_NOOP("Sorry, failed to name user directory.");
             }
         }
     }
@@ -599,9 +609,15 @@ bool Oobe::setUserInfo()
     }
 
     // fix the ownership, demo=newuser
-    if (!proc.exec("chown", {"-R", "demo:demo", dpath})) {
-        failure = tr("Sorry, failed to set ownership of user directory.");
-        return false;
+    ok = proc.exec("chown", {"-R", "demo:demo", dpath});
+    if (ok) {
+        // Set permissions according to /etc/adduser.conf
+        QSettings addusercfg("/etc/adduser.conf", QSettings::NativeFormat);
+        mode_t mode = addusercfg.value("DIR_MODE", "0700").toString().toUInt(&ok, 8);
+        if (ok) ok = (chmod(dpath.toUtf8().constData(), mode) == 0);
+    }
+    if (!ok) {
+        throw QT_TR_NOOP("Failed to set ownership or permissions of user directory.");
     }
 
     // change in files
@@ -609,6 +625,8 @@ bool Oobe::setUserInfo()
     replaceStringInFile("demo", username, rootpath + "/etc/gshadow");
     replaceStringInFile("demo", username, rootpath + "/etc/passwd");
     replaceStringInFile("demo", username, rootpath + "/etc/shadow");
+    replaceStringInFile("demo", username, rootpath + "/etc/subuid");
+    replaceStringInFile("demo", username, rootpath + "/etc/subgid");
     replaceStringInFile("demo", username, rootpath + "/etc/slim.conf");
     replaceStringInFile("demo", username, rootpath + "/etc/slimski.local.conf");
     replaceStringInFile("demo", username, rootpath + "/etc/lightdm/lightdm.conf");
@@ -616,7 +634,7 @@ bool Oobe::setUserInfo()
     replaceStringInFile("demo", username, rootpath + "/root/.gtkrc-2.0");
     if (gui.checkAutoLogin->isChecked()) {
         replaceStringInFile("#auto_login", "auto_login", rootpath + "/etc/slim.conf");
-        replaceStringInFile("#default_user ", "default_user ", rootpath + "/etc/slim.conf"); 
+        replaceStringInFile("#default_user ", "default_user ", rootpath + "/etc/slim.conf");
         replaceStringInFile("#autologin_enabled", "autologin_enabled", rootpath + "/etc/slimski.local.conf");
         replaceStringInFile("#default_user ", "default_user ", rootpath + "/etc/slimski.local.conf");
         replaceStringInFile("User=", "User=" + username, rootpath + "/etc/sddm.conf");
@@ -630,8 +648,6 @@ bool Oobe::setUserInfo()
         replaceStringInFile("User=.*", "User=", rootpath + "/etc/sddm.conf");
     }
     proc.exec("touch", {rootpath + "/var/mail/" + username});
-
-    return true;
 }
 
 void Oobe::resetBlueman()
@@ -654,7 +670,7 @@ void Oobe::timeAreaIndexChanged(int index)
     if (index < 0 || index >= gui.comboTimeArea->count()) return;
     const QString &area = gui.comboTimeArea->itemData(index).toString();
     gui.comboTimeZone->clear();
-    for (const QString &zone : timeZones) {
+    for (const QString &zone : qAsConst(timeZones)) {
         if (zone.startsWith(area)) {
             QString text(QString(zone).section('/', 1));
             text.replace('_', ' ');
